import os


class AVLTree:
    """
    This class implements the AVL tree data structure.
    For more information, please refer to: https://en.wikipedia.org/wiki/AVL_tree
    """

    class AVLNode:
        """
        Represents a node inside the AVL tree
        """

        def __init__(self, value, **kwargs):
            """
            Constructor
            :param value: The value represented by this node
            :param kwargs: other arguments for ancestors
            """
            super().__init__(**kwargs)
            self.value = value
            self.left = None
            self.right = None
            self.max_depth = 1

        def to_string(self):
            return self.value

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.root = None
        self.size = 0

    def add_node(self, node):
        if self.root is None:
            self.root = node
            self.size = 1

        else:
            path = list()
            current = self.root
            parent = None
            while current is not None:
                if node.value == current.value:  # the value is already there
                    return
                elif node.value < current.value:
                    parent = current
                    current = current.left
                else:
                    parent = current
                    current = current.right

                path.append(parent)

            if node.value < parent.value:
                parent.left = node
            else:
                parent.right = node

            path_depth = 1
            self.size += 1

            for i in range(len(path) - 1, -1, -1):
                path_depth += 1
                if path[i].max_depth >= path_depth:  # the other subtree has the same depth, no need for rebalancing
                    break
                else:
                    path[i].max_depth += 1
                    self.rebalance(path[i])  # we need to re-balance the tree

    def rebalance(self, node):
        """
        Re-balances the tree starting at the given node
        :param node: The lowest node that violates the balancing rulss
        :return: nothing
        """
        pass  # re-balancing not implemented for now

    def to_string(self, node=None, depth=0):
        """
        Converts the representation to string
        :return: The string representation of the tree, for pretty-printing
        """
        return self._print_node_(self.root, 0)

    def _print_node_(self, node, depth):
        if node is None:
            return ''
        else:
            res = str()
            res += str(node.value)
            res += '  ' + self._print_node_(node.right, depth + 1)
            res += ((depth + 1) * 2) * '  ' + self._print_node_(node.left, depth + 1)
            res += os.linesep

            return res

    def __iter__(self):
        yield from self._iterate_(self.root)

    def _iterate_(self, node):
        items = list()
        if node is None:
            return list()

        else:
            items.extend(self._iterate_(node.left))
            items.append(node.value)
            items.extend(self._iterate_(node.right))
            return items

    def __getitem__(self, item):  # note that this is not optimal, since it always goes through the entire tree
        # generally - indexing to non-constant trees is not a good idea
        return [i for i in self.__iter__()][item]

    def __call__(self):
        return self.root

    def __len__(self):
        return self.size


tree = AVLTree()

input_data = '43 25 66 2 5 9 7 52 65 78 39 44 25 66 82 94'

for i in input_data.split():
    tree.add_node(AVLTree.AVLNode(int(i)))

for i in range(len(tree)):
    print(tree[i])

print(tree().to_string())

if tree:
    print(1)
else:
    print(0)

if AVLTree():
    print(1)
else:
    print(0)